class QuickvoteController < ApplicationController
  layout 'hc'
  model :quick_voter
  model :quick_vote
  model :vote
  model :election

  #############################################################
  # the following methods pertain to creating quickvotes
  #############################################################

  def create
    if params[:quickvote] 
      @quickvote = QuickVote.new(params[:quickvote])

      # store the candidate grabbed through ajax and stored in flash
      @quickvote.candidatelist = flash[:candlist]

      # try to save, if it fails, show the page again (the flash should
      # still be intact
      if @quickvote.save
        @quickvote = @quickvote.reload
        render :action => 'success'
      else
        flash.keep(:candlist)
      end 

    else
      # if we don't have a quickvote param, it means that the person
      # here has not been hitting this page and we can clear any
      # candlist in the flash
      flash.delete(:candlist) if flash.has_key?(:candlist)
    end
  end

  def add_candidate
    candidate_name = params[:ajax][:newcandidate]
    if flash.has_key?(:candlist) and flash[:candlist].instance_of?(Array) 
      flash[:candlist] << candidate_name
    else
      flash[:candlist] = [ candidate_name ]
    end
    flash.keep(:candlist)
    render_partial 'candidate_list'
  end
 
  #############################################################
  # the following methods pertain to *voting* in the quickvotes
  #############################################################

  def index
    @election = QuickVote.find_all(["name = ?", params[:votename]])[0]
    
    # if the person has specified an election, we show them the voting
    # page. otherwise, we redirect back to main the page
    if @election

      # look to see that the voter has been created and has voted in
      # this election, and has confirmed their vote
      @voter = QuickVoter.find_all(["session_id = ? and election_id = ?",
                                  session.session_id, @election.id])[0]

      # if the voter has not voted we destroy them
      if @voter and not @voter.voted?
        @voter.destroy
	@voter = nil
      end

      # if the voter does not exist or as has been destroyed, lets
      # create a new one
      unless @voter
        # create a new voter and populate it
        @voter = QuickVoter.new
        @voter.election = QuickVote.find_all( [ "name = ?", params[:votename] ] )[0]
        @voter.session_id = session.session_id

	# create new vote and make it the defaulted sorted list
        @voter.vote = Vote.new
	@voter.save
	@voter.vote.set_defaults!
	@voter.reload
      end
    else
      redirect_to :controller => 'site'
    end
  end

  def confirm
    # we need the election to verify that we have the right voter
    election = QuickVote.find_all( [ "name = ?", params[:votename] ] )[0]

    # find out who the voter is for this election
    @voter = QuickVoter.find_all(["session_id = ? and election_id = ?", 
                                  session.session_id, election.id])[0]

    if not @voter
      # we have not seen this  voter before. something is wrong, try
      # again
      redirect_to quickvote_url( :votename => params[:votename] ) 
      
    elsif @voter.voted?
      # this person has already voted, we try again
      flash[:notice] = "You have already voted!"
      redirect_to quickvote_url( :votename => params[:votename] )
      
    else
      # record the ip address for posterity
      @voter.ipaddress = request.env["HTTP_X_FORWARDED_FOR"]
      @voter.save
      
      # toggle the confirmation bit
      @voter.vote.confirm!
      @voter.reload
      render :action => 'thanks'
    end
  end
 
  def change
    voter = QuickVoter.find_all(["session_id = ?", session.session_id])[0]
    voter.destroy
    redirect_to quickvote_url( :votename => params[:votename] )
  end

  def sort_candidates
    @vote = Vote.find(params[:id])

    @vote.rankings.each do |ranking|
      ranking.rank = params['rankings-list'].index(ranking.candidate.id.to_s) + 1
      ranking.save
    end
    render :nothing => true
  end
		

  ###############################################################
  # the following method pertains to displaying the results of a
  # quickvote
  ###############################################################

  def results
    @election = QuickVote.find_all( ["name = ?", params[:votename]] )[0]

    # initalize the tallies to empty arrays
    preference_tally = Array.new
    plurality_tally = Array.new
    approval_tally = Array.new

    @election.voters.each do |voter|
      # skip if the voter has not voted or has an unconfirmed vote
      next unless voter.voted?
      
      plurality_tally << voter.vote.rankings.sort[0].candidate.id
      approval_tally << voter.vote.rankings.sort[0..1].collect \
        { |ranking| ranking.candidate.id }
      preference_tally << voter.vote.rankings.sort.collect \
        { |ranking| ranking.candidate.id }
    end
 
    @plurality_result = PluralityVote.new(plurality_tally).result
    @approval_result = ApprovalVote.new(approval_tally).result
    @condorcet_result = PureCondorcetVote.new(preference_tally).result
    @ssd_result = CloneproofSSDVote.new(preference_tally).result
    @borda_result = BordaVote.new(preference_tally).result
    #@runoff_result = InstantRunoffVote.new(preference_tally).result
    #@runoff_results = PluralityVote.new(preference_tally).result


    @candidates = {} 
    @election.candidates.each {|c| @candidates[c.id] = c}
  end

end
